import openai
import json
from termcolor import colored

# define your OpenAI API key
# openai.api_key = "YOUR_API_KEY"

model = "gpt-3.5-turbo-16k-0613"
# model = "gpt-4-0613"

def summarize_text(text, model):
    response = openai.ChatCompletion.create(
        model = model,
        messages = [
            {"role": "user", "content": f"summarize: {text}"},
        ],
        stream = True,
        functions = [
            {
                "name": "summarize_document",
                "description": "Summarizes a document by returning a summary, a python list of important points, and a python list of keywords",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "summary": {
                            "type": "string",
                            "description": "a short summary of the document."
                        },
                        "important_points": {
                            "type": "array",
                            "items": {
                                "type": "string",
                                "description": "a python list of important points in the document."
                            }
                        },
                        "keywords": {
                            "type": "array",
                            "items": {
                                "type": "string",
                                "description": "a python list of keywords in the document."
                            }
                        }
                    },
                    "required": ["summary", "important_points", "keywords"]
                }
            }
        ],
        function_call = {"name": "summarize_document"} # can be "auto" or "none"
    )

    responses = ''
    for chunk in response:
        # print(chunk)
        if chunk["choices"][0]["delta"].get("function_call"):
            chunk = chunk["choices"][0]["delta"]
            # print(chunk)
            token = chunk["function_call"]["arguments"]
            responses += token
            print(colored(token, "green"), end='', flush=True)

    # assign all arguments to variables
    summary, important_points, keywords = json.loads(responses).values()

    return summary, important_points, keywords

# read the contents of sample.txt
with open('sample.txt', 'r', encoding="utf-8", errors="ignore") as file:
    sample = file.read().replace('\n', '')

summary, important_points, keywords = summarize_text(sample, model)

print("\n")
print(colored(f"Summary: {summary}", "blue")) 
print("\n")
print(colored(f"Important Points: {important_points}", "yellow"))
print("\n")
print(colored(f"Keywords: {keywords}", "magenta"))